* Neo Geo/CD Frog Feast (c) RasterSoft

	.include "Includes/Defines.inc"

	.globl	SetupTilemap
	.globl	LoadTilemap
	.globl	FillTilemap
	.globl	LoadPalette
	.globl	ClearText
	.globl	ReadJoystickA
	.globl	ReadJoystickB
	.globl	SetPaletteRed
	.globl	SetPaletteGreen
	.globl	SetPaletteBlue
	.globl	DrawNumber
	.globl	DrawString
	.globl	WaitVBlank
	.globl	VBlank
	.globl	RenderSprites
	.globl	Seed
	.globl	Random

	.extern	SpritesChanged
	.extern	SpriteControl
	.extern	SpriteBlock

* void SetupTilemap(DWORD y, DWORD Tile);
* D0 - TileY
* D1 - Starting tile number
SetupTilemap:
	.set	_ARGS, 4
	move.l	_ARGS(a7), d0
	move.l	_ARGS+4(a7), d1

	* Save data registers
	movem.l d0-d2, -(sp)

*	move.w	ScrollPos, d2
*	lsl.w	#7, d2
*	add.w	d2, d0

	* Starting tile number
	move.w	d1, VRAM_ADDR

	* Increment VRAM pos by 0x200
	move.w	#0x0200, VRAM_INCR

	* 0x0FFF = Full size for sprite
	move.w	#FULLZOOM, VRAM_DATA

	* Starting y position of the tilemap
	move.w	d0, VRAM_DATA

	* 0xE sprite in the bank
	* Ypos = 0xFC0E >> 7 = 504
	* Xpos = 0x440 >> 7 = 8
	move.w	#0x440, VRAM_DATA

	* Sprite width - 1
	move.w	#MAPWIDTH - 1, d0

SetupTilemapLoop:
	add.w	#1, d1
	move.w	d1, VRAM_ADDR

	* Full size/zoom
	move.w	#FULLZOOM, VRAM_DATA

	* Bit 6 = 1 : next sprite at right
	move.w	#0x40, VRAM_DATA

	dbra.w	d0, SetupTilemapLoop

	* Restore data registers
	movem.l (sp)+, d0-d2

	rts

* void LoadTilemap(DWORD *Source, DWORD Dest, DWORD Width, DWORD Height, DWORD Tile);
* A0 - VRAM Location
* A1 - Map pointer
* D0 - Width
* D1 - Height
* D2 - Starting sprite #
LoadTilemap:
	.set	_ARGS, 4
	move.l	_ARGS(a7), a1
	move.l	_ARGS+4(a7), a0
	move.l	_ARGS+8(a7), d0
	move.l	_ARGS+12(a7), d1
	move.l	_ARGS+16(a7), d2

	* Adjust height
	sub.l	#1, d1

	* Save data registers
	movem.l d0-d4/a0-a2, -(sp)

	move.l	a1, a2

	* Location to write a0
	move.w	a0, VRAM_ADDR

	* VRAM increment by 1
	move.w	#0x0001, VRAM_INCR

	* Save Height
	move.w	d1, d3

LoadTilemapOuterLoop:
	move.w	d3, d1

LoadTilemapInnerLoop:
	* Transfer tile number
	move.w	(a1)+, VRAM_DATA

	* Transfer attribute
	move.w	(a1)+, VRAM_DATA

	dbra.w	d1, LoadTilemapInnerLoop

	* Skip to next collumn
   	add.l	#MAPHEIGHT*4, a2
	move.l	a2, a1

	* X + 1
	add.w	#64, a0

	* Location to write a0
	move.w	a0, VRAM_ADDR

	dbra.w	d0, LoadTilemapOuterLoop

	* Restore data registers
	movem.l (sp)+, d0-d4/a0-a2

	rts

* void FillTilemap(DWORD Dest, DWORD Width, DWORD Height, DWORD Tile);
* A0 - VRAM Location
* D0 - Width
* D1 - Height
* D2 - Starting sprite #
FillTilemap:
	.set	_ARGS, 4
	move.l	_ARGS(a7), a0
	move.l	_ARGS+4(a7), d0
	move.l	_ARGS+8(a7), d1
	move.l	_ARGS+12(a7), d2

	* Adjust height
	sub.l	#1, d1

	* Save data registers
	movem.l	d0-d4/a0-a2, -(sp)

	* VRAM increment by 1
	move.w	#0x0001, VRAM_INCR

	* Location to write a0
	move.w	a0, VRAM_ADDR

	* Save Height
	move.w	d1, d3

FillTilemapOuterLoop:
	move.w	d3, d1

FillTilemapInnerLoop:
	* Transfer tile number
	move.w	#0x0000, VRAM_DATA

	* Transfer attribute
	move.w	#0x0000, VRAM_DATA

	dbra.w	d1, FillTilemapInnerLoop

	* Skip to next collumn
   	add.l	#MAPHEIGHT*4, a2
	move.l	a2, a1

	* X + 1
	add.w	#64, a0

	* Location to write a0
	move.w	a0, VRAM_ADDR

	dbra.w	d0, FillTilemapOuterLoop

	* Restore data registers
	movem.l (sp)+, d0-d4/a0-a2

	rts

* void LoadPalette(WORD *Source, DWORD Dest, DWORD Length);
* A0 = ptr to NeoGeo palette
* A1 = ptr to our palette
* D0 = palette size
LoadPalette:
	.set	_ARGS, 4
	move.l	_ARGS(a7), a1
	move.l	_ARGS+4(a7), a0
	move.l	_ARGS+8(a7), d0

	* Backup data registers
	movem.l	d0/a0-a1, -(sp)

	* Each pal = 16 words so palette number = d0 >> 5
	asr.w	#5, d0

* Copy 16 color per palettes
SetupPaletteLoop:
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+

	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+

	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+

	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+
	move.w	(a1)+, (a0)+

	dbra	d0, SetupPaletteLoop

	* Restore data registers
	movem.l (sp)+, d0/a0-a1

	rts

* void RenderSprites()
RenderSprites:
	movem.l	d0-d7/a0-a1, -(sp)

	* See if any sprites have changed
	cmp.l	#1, SpritesChanged
	bne		EndRenderSprites

	* Reset changed flag
	move.l	#0, SpritesChanged

	jsr		SpriteClear

	* Load pointer to Sprite Control Structure
	lea		SpriteControl, a0

	move.w	#0x0200, VRAM_INCR

	* Load starting sprite number
	move.w	#SPRITESPR, d4

	* Load number of sprites
	move.w	#MAXSPRITES-1, d5

RenderSpritesLoop1:
	* Load used flag
	move.w	(a0)+, d0

	* Load zoom
	move.w	(a0)+, d1

	* Load y
	move.w	(a0)+, d2

	* Load x
	move.w	(a0)+, d3

	* Check if sprite is used
	cmp.w	#0, d0
	beq.s	EndRenderLoop1

	* Add sprite to system
	move.w	d4, VRAM_ADDR
	add.w	#1, d4

	move.w	d1, VRAM_DATA
	move.w	d2, VRAM_DATA
	move.w	d3, VRAM_DATA

EndRenderLoop1:
	dbra.w	d5, RenderSpritesLoop1

	lea		SpriteControl, a0
	lea		SpriteBlock, a1

	move.w	#0x0001, VRAM_INCR
	move.w	#VRAMSPR, d4

	move.w	#MAXSPRITES-1, d5

RenderSpritesLoop2:
	move.w	(a0), d0
	move.w	(a1)+, d1
	move.w	(a1)+, d2

	add.l	#8, a0

	cmp.w	#0, d0
	beq.s	EndRenderLoop2

	move.w	d4, VRAM_ADDR
	add.w	#0x40, d4

	move.w	d1, VRAM_DATA
	move.w	d2, VRAM_DATA

EndRenderLoop2:
	dbra.w	d5, RenderSpritesLoop2

EndRenderSprites:
	movem.l (sp)+, d0-d7/a0-a1

	rts

SpriteClear:
	move.w	#MAXSPRITES, d1
	move.w	#SPRITESPR, d2
	move.w	#0x0200, VRAM_INCR

SpriteClearLoop:
	move.w	d2, VRAM_ADDR
	add.w	#1, d2

	move.w	#0x0000, VRAM_DATA
	move.w	#0x0000, VRAM_DATA
	move.w	#0x0000, VRAM_DATA

	dbra.w	d1, SpriteClearLoop

	rts

* void ClearText();
ClearText:
	movem.l d0-d2, -(sp)
	
	move.l	#32*40, d0

	* Clear value
	move.w	#CHARNUMBER, d1

	* 0x7000 is the FIX plane
	move.w	#0x7000, d2
	
ClearTextLoop:
	* Set write location in VRAM
	move.w	d2, VRAM_ADDR			
	
	* Write character and palette
	move.w	d1, VRAM_DATA

	add.w	#1, d2

	dbra	d0, ClearTextLoop

	movem.l (sp)+, d0-d2

	rts

* DWORD ReadJoystickA();
ReadJoystickA:
	clr.l	d0
	move.w	JoyStickValA, d0

	rts

* DWORD ReadJoystickB();
ReadJoystickB:
	clr.l	d0
	move.w	JoyStickValB, d0

	rts

* void SetPaletteRed(DWORD Dest, DWORD Red);
* A0 - Palette pointer
* D1 - Color
SetPaletteRed:
	.set	_ARGS, 4
	move.l	_ARGS(a7), a0
	move.l	_ARGS+4(a7), d1

	movem.l	d0-d1, -(sp)

	move.w	(a0), d0
	and.w	#0x00FF, d0
	and.w	#0x0F, d1
	asl.w	#8, d1
	or.w	d1, d0
	move.w	d0, (a0)

	movem.l (sp)+, d0-d1

	rts

* void SetPaletteGreen(DWORD Dest, DWORD Green);
* A0 - Palette pointer
* D1 - Color
SetPaletteGreen:
	.set	_ARGS, 4
	move.l	_ARGS(a7), a0
	move.l	_ARGS+4(a7), d1

	movem.l	d0-d1, -(sp)

	move.w	(a0), d0
	and.w	#0x0F0F, d0
	and.w	#0x0F, d1
	asl.w	#4, d1
	or.w	d1, d0
	move.w	d0, (a0)

	movem.l (sp)+, d0-d1

	rts

* void SetPaletteBlue(DWORD Dest, DWORD Blue);
* A0 - Palette pointer
* D1 - Color
SetPaletteBlue:
	.set	_ARGS, 4
	move.l	_ARGS(a7), a0
	move.l	_ARGS+4(a7), d1

	movem.l	d0-d1, -(sp)

	move.w	(a0), d0
	and.w	#0x0FF0, d0
	and.w	#0x0F, d1
	or.w	d1, d0
	move.w	d0, (a0)

	movem.l (sp)+, d0-d1

	rts

* void DrawNumber(DWORD Value, DWORD Palette, DWORD x, DWORD y);
* D1 - Value, D2 - Palette, D3 - x, D4 - y, D5 - Location
DrawNumber:
	.set	_ARGS, 4

	* Get value
	move.l	_ARGS(a7), d1		
	
	* Get palette
	move.l	_ARGS+4(a7), d2		

	* Get x
	move.l	_ARGS+8(a7), d3

	* Get y
	move.l	_ARGS+12(a7), d4		

	* Shift palette 12 bits
	asl.l	#8, d2
	asl.l	#4, d2

	* Divide score by 10
	move.l	#10, -(sp)			
	move.l	d1, -(sp)
	jsr		ldiv
	addq.w	#8, sp

	* Point to correct character
	add.l	#48, d1

	jsr		DrawDigit

	* See if there is a significant digit
	cmp.b	#0, d0
	beq		NoDrawNumber

	* Draw 1 position to the left
	sub.l	#1, d3

	* Get upper digit
	move.l	d0, d1

	* Point to correct character
	add.l	#48, d1

	jsr		DrawDigit

NoDrawNumber:
	rts

* D1 - Number, D2 - Palette, D3 - x, D4 - y
DrawDigit:
	movem.l d0-d5, -(sp)

	* Point to character data
	add.l	#CHARNUMBER, d1

	* Multiply x by 32
	asl.l	#5, d3				

	* Add in y position
	add.l	d4, d3				

	* 0x7000 is the FIX plane
	add.l	#0x7000, d3			
	
	or.l	d2, d1

	* Set write location in VRAM
	move.w	d3, VRAM_ADDR			
	
	* Write character and palette
	move.w	d1, VRAM_DATA

	movem.l (sp)+, d0-d5

	rts

* void DrawString(char *String, DWORD Palette, DWORD x, DWORD y);
* A0 - String, D2 - Palette, D3 - x, D4 - y, D5 - Location
DrawString:
	.set	_ARGS, 4

	* Get string
	move.l	_ARGS(a7), a0
	
	* Get palette
	move.l	_ARGS+4(a7), d2		

	* Get x
	move.l	_ARGS+8(a7), d3

	* Get y
	move.l	_ARGS+12(a7), d4		

	* Shift palette 12 bits
	asl.l	#8, d2
	asl.l	#4, d2

	clr.l	d1

ContinueDrawString:
	move.b	(a0)+, d1

	cmp.b	#0, d1
	beq.s	EndDrawString

	jsr		DrawChar

	add.l	#1, d3

	bra.s	ContinueDrawString

EndDrawString:
	rts

* D1 - Character, D2 - Palette, D3 - x, D4 - y, D5 - Location
DrawChar:
	movem.l d0-d3, -(sp)

	* Point to character data
	add.l	#CHARNUMBER, d1

	* Multiply x by 32
	asl.l	#5, d3				

	* Add in y position
	add.l	d4, d3				

	* 0x7000 is the FIX plane
	add.l	#0x7000, d3			
	
	or.l	d2, d1

	* Set write location in VRAM
	move.w	d3, VRAM_ADDR			
	
	* Write character and palette
	move.w	d1, VRAM_DATA

	movem.l (sp)+, d0-d3

	rts

* void WaitVBlank();
WaitVBlank:
	cmp.w	#1, VertBlank
	bne		WaitVBlank

	clr.w	VertBlank

	rts

* VBlank interrupt routine
VBlank:
	* Bit 7 = 1 when NeoGeo logo is displayed
	tst.b	0x10FD80
	bmi.s	DisplayedLogo

	* NeoGeo logo display function
	jmp		0xC00438

DisplayedLogo:
	movem.l	d0-d7/a0-a6,-(sp)

	* Reset watchdog
	move.b	d0, 0x300001

	jsr		0xC0044A

	* Read joystick A
	move.b	STARTBUTTONS, d0
	and.w	#0x03, d0
	lsl.w	#8, d0
	move.b	JOYSTICKA, d0
	not.w	d0
	move.w	d0, JoyStickValA

	* Read joystick B
	move.b	STARTBUTTONS, d0
	and.w	#0x0C, d0
	lsl.w	#6, d0
	move.b	JOYSTICKB, d0
	not.w	d0
	move.w	d0, JoyStickValB

	* Render sprites
	jsr		RenderSprites

	* Vertical Blank occured
	move.w	#1, VertBlank

	movem.l	(sp)+, d0-d7/a0-a6

	* IRQ 2 acknowledge
	move.w	#4, 0x3C000C

	rte

* D0 - Seed
Seed:
	* user seed in d0 (d1 too)
	add.l	d0, d1
	movem.l	d0-d1, RandomSeed

* D0 - Limit
LongRnd:
	movem.l d2-d3, -(sp)
	* d0=LSB's, d1=MSB's of random number
	movem.l RandomSeed, d0-d1
	* ensure upper 59 bits are an...
	andi.b	#0x0E, d0
	* ...odd binary number
	ori.b	#0x20, d0
	move.l	d0, d2
	move.l	d1, d3
	* accounts for 1 of 17 left shifts
	add.l	d2, d2
	* [d2/d3] = RND*2
	addx.l	d3, d3
	add.l	d2, d0
	* [d0/d1] = RND*3
	addx.l	d3, d1
	* shift [d2/d3] additional 16 times
	swap	d3
	swap	d2
	move.w	d2, d3
	clr.w	d2
	* add to [d0/d1]
	add.l	d2, d0
	addx.l	d3, d1
	* save for next time through
	movem.l d0-d1, RandomSeed
	* most random part to d0
	move.l	d1, d0
	movem.l (sp)+, d2-d3

	rts

* void Random(DWORD Limit);
* D0 - Limit
Random:
	.set	_ARGS, 4

	* Get value
	move.l	_ARGS(a7), d0

	move.l	d2, -(sp)

	* save upper limit
	move.w	d0, d2
	cmp 	#0, d0

	* range of 0 returns 0 always
	beq.s	Zero

	* get a longword random number
	bsr 	LongRnd

	* use upper word (it's most random)
	clr.w	d0
	swap	d0

	* divide by range...
	divu.w	d2, d0

	* ...and use remainder for the value
	clr.w	d0

	* result in d0.w
	swap	d0

Zero:
	move.l	(sp)+, d2

	rts

ldiv:
	move.l  4(a7),d0
	bpl     ld1
	neg.l   d0
ld1:
	move.l  8(a7),d1
	bpl     ld2
	neg.l   d1
	eor.b   #0x80,4(a7)
ld2:
	bsr     i_ldiv          /* d0 = d0/d1 */
	tst.b   4(a7)
	bpl     ld3
	neg.l   d0
ld3:
	rts

lmul:
	move.l  4(a7),d0
	bpl     lm1
	neg.l   d0
lm1:
	move.l  8(a7),d1
	bpl     lm2
	neg.l   d1
	eor.b   #0x80,4(a7)
lm2:
	bsr     i_lmul          /* d0 = d0*d1 */
	tst.b   4(a7)
	bpl     lm3
	neg.l   d0

lm3:
	rts

*
* A in d0, B in d1, return A*B in d0
*
i_lmul:
	move.l  d3,a2           /* save d3 */
	move.w  d1,d2
	mulu    d0,d2           /* d2 = Al * Bl */

	move.l  d1,d3
	swap    d3
	mulu    d0,d3           /* d3 = Al * Bh */

	swap    d0
	mulu    d1,d0           /* d0 = Ah * Bl */

	add.l   d3,d0           /* d0 = (Ah*Bl + Al*Bh) */
	swap    d0
	clr.w   d0              /* d0 = (Ah*Bl + Al*Bh) << 16 */

	add.l   d2,d0           /* d0 = A*B */
	move.l  a2,d3           /* restore d3 */
	rts
*
*A in d0, B in d1, return A/B in d0, A%B in d1
*
i_ldiv:
	tst.l   d1
	bne     nz1

*       divide by zero
*	divu    #0,d0           /* cause trap */
	move.l  #0x80000000,d0
	move.l  d0,d1
	rts
nz1:
	move.l  d3,a2           /* save d3 */
	cmp.l   d1,d0
	bhi     norm
	beq     is1
*       A<B, so ret 0, rem A
	move.l  d0,d1
	clr.l   d0
	move.l  a2,d3           /* restore d3 */
	rts
*       A==B, so ret 1, rem 0
is1:
	moveq.l #1,d0
	clr.l   d1
	move.l  a2,d3           /* restore d3 */
	rts
*       A>B and B is not 0
norm:
	cmp.l   #1,d1
	bne     not1
*       B==1, so ret A, rem 0
	clr.l   d1
	move.l  a2,d3           /* restore d3 */
	rts
*  check for A short (implies B short also)
not1:
	cmp.l   #0xffff,d0
	bhi     slow
*  A short and B short -- use 'divu'
	divu    d1,d0           /* d0 = REM:ANS */
	swap    d0              /* d0 = ANS:REM */
	clr.l   d1
	move.w  d0,d1           /* d1 = REM */
	clr.w   d0
	swap    d0
	move.l  a2,d3           /* restore d3 */
	rts
* check for B short
slow:
	cmp.l   #0xffff,d1
	bhi     slower
* A long and B short -- use special stuff from gnu
	move.l  d0,d2
	clr.w   d2
	swap    d2
	divu    d1,d2           /* d2 = REM:ANS of Ahi/B */
	clr.l   d3
	move.w  d2,d3           /* d3 = Ahi/B */
	swap    d3

	move.w  d0,d2           /* d2 = REM << 16 + Alo */
	divu    d1,d2           /* d2 = REM:ANS of stuff/B */

	move.l  d2,d1
	clr.w   d1
	swap    d1              /* d1 = REM */

	clr.l   d0
	move.w  d2,d0
	add.l   d3,d0           /* d0 = ANS */
	move.l  a2,d3           /* restore d3 */
	rts
*       A>B, B > 1
slower:
	move.l  #1,d2
	clr.l   d3
moreadj:
	cmp.l   d0,d1
	bhs     adj
	add.l   d2,d2
	add.l   d1,d1
	bpl     moreadj
* we shifted B until its >A or sign bit set
* we shifted #1 (d2) along with it
adj:
	cmp.l   d0,d1
	bhi     ltuns
	or.l    d2,d3
	sub.l   d1,d0
ltuns:
	lsr.l   #1,d1
	lsr.l   #1,d2
	bne     adj
* d3=answer, d0=rem
	move.l  d0,d1
	move.l  d3,d0
	move.l  a2,d3           /* restore d3 */

	rts
